@@ -1,7 +1,40 @@
+/*
+ * Copyright 2004 Eric Anholt
+ * All Rights Reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
+ *
+ * Authors:
+ *    Eric Anholt <anholt@FreeBSD.org>
+ *    Hui Yu <hyu@ati.com>
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <string.h>
+
+#ifdef USE_XAA
+
 #include "dixstruct.h"
 
 #include "xaa.h"
@@ -10,11 +43,6 @@
 #ifndef RENDER_GENERIC_HELPER
 #define RENDER_GENERIC_HELPER
 
-static void RadeonInit3DEngineMMIO(ScrnInfoPtr pScrn);
-#ifdef XF86DRI
-static void RadeonInit3DEngineCP(ScrnInfoPtr pScrn);
-#endif
-
 struct blendinfo {
 	Bool dst_alpha;
 	Bool src_alpha;
@@ -228,24 +256,6 @@ ATILog2(int val)
 	return bits - 1;
 }
 
-static void RadeonInit3DEngine(ScrnInfoPtr pScrn)
-{
-    RADEONInfoPtr info = RADEONPTR (pScrn);
-
-#ifdef XF86DRI
-    if (info->directRenderingEnabled) {
-	RADEONSAREAPrivPtr pSAREAPriv;
-
-	pSAREAPriv = DRIGetSAREAPrivate(pScrn->pScreen);
-	pSAREAPriv->ctxOwner = DRIGetContext(pScrn->pScreen);
-	RadeonInit3DEngineCP(pScrn);
-    } else
-#endif
-	RadeonInit3DEngineMMIO(pScrn);
-
-    info->RenderInited3D = TRUE;
-}
-
 static void
 RemoveLinear (FBLinearPtr linear)
 {
@@ -273,11 +283,17 @@ AllocateLinear (
    ScrnInfoPtr pScrn,
    int sizeNeeded
 ){
-    RADEONInfoPtr  info       = RADEONPTR(pScrn);
+   RADEONInfoPtr  info       = RADEONPTR(pScrn);
+   int cpp = info->CurrentLayout.bitsPerPixel / 8;
 
    info->RenderTimeout = currentTime.milliseconds + 30000;
    info->RenderCallback = RenderCallback;
 
+   /* XAA allocates in units of pixels at the screen bpp, so adjust size
+    * appropriately.
+    */
+   sizeNeeded = (sizeNeeded + cpp - 1) / cpp;
+
    if (info->RenderTex) {
 	if (info->RenderTex->size >= sizeNeeded)
 	   return TRUE;
@@ -301,29 +317,27 @@ static Bool RADEONSetupRenderByteswap(ScrnInfoPtr pScrn, int tex_bytepp)
 {
     RADEONInfoPtr info = RADEONPTR(pScrn);
     unsigned char *RADEONMMIO = info->MMIO;
+    CARD32 swapper = info->ModeReg.surface_cntl;
+
+    swapper &= ~(RADEON_NONSURF_AP0_SWP_16BPP | RADEON_NONSURF_AP1_SWP_16BPP |
+		 RADEON_NONSURF_AP0_SWP_32BPP | RADEON_NONSURF_AP1_SWP_32BPP);
 
     /* Set up byte swapping for the framebuffer aperture as needed */
     switch (tex_bytepp) {
     case 1:
-	OUTREG(RADEON_SURFACE_CNTL, info->ModeReg.surface_cntl &
-				    ~(RADEON_NONSURF_AP0_SWP_32BPP
-				    | RADEON_NONSURF_AP0_SWP_16BPP));
 	break;
     case 2:
-	OUTREG(RADEON_SURFACE_CNTL, (info->ModeReg.surface_cntl &
-				     ~RADEON_NONSURF_AP0_SWP_32BPP)
-				   | RADEON_NONSURF_AP0_SWP_16BPP);
+	swapper |= RADEON_NONSURF_AP0_SWP_16BPP | RADEON_NONSURF_AP1_SWP_16BPP;
 	break;
     case 4:
-	OUTREG(RADEON_SURFACE_CNTL, (info->ModeReg.surface_cntl &
-				     ~RADEON_NONSURF_AP0_SWP_16BPP)
-				   | RADEON_NONSURF_AP0_SWP_32BPP);
+	swapper |= RADEON_NONSURF_AP0_SWP_32BPP | RADEON_NONSURF_AP1_SWP_32BPP;
 	break;
     default:
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "%s: Don't know what to do for "
 		   "tex_bytepp == %d!\n", __func__, tex_bytepp);
 	return FALSE;
     }
+    OUTREG(RADEON_SURFACE_CNTL, swapper);
     return TRUE;
 }
 
@@ -357,61 +371,6 @@ static void RADEONRestoreByteswap(RADEONInfoPtr info)
 #endif
 #endif
 
-
-static void FUNC_NAME(RadeonInit3DEngine)(ScrnInfoPtr pScrn)
-{
-    RADEONInfoPtr  info       = RADEONPTR(pScrn);
-    ACCEL_PREAMBLE();
-
-    if (info->ChipFamily >= CHIP_FAMILY_R300) {
-	/* Unimplemented */
-    } else if ((info->ChipFamily == CHIP_FAMILY_RV250) || 
-	       (info->ChipFamily == CHIP_FAMILY_RV280) || 
-	       (info->ChipFamily == CHIP_FAMILY_RS300) || 
-	       (info->ChipFamily == CHIP_FAMILY_R200)) {
-
-	BEGIN_ACCEL(7);
-        if (info->ChipFamily == CHIP_FAMILY_RS300) {
-            OUT_ACCEL_REG(R200_SE_VAP_CNTL_STATUS, RADEON_TCL_BYPASS);
-        } else {
-            OUT_ACCEL_REG(R200_SE_VAP_CNTL_STATUS, 0);
-        }
-	OUT_ACCEL_REG(R200_PP_CNTL_X, 0);
-	OUT_ACCEL_REG(R200_PP_TXMULTI_CTL_0, 0);
-	OUT_ACCEL_REG(R200_SE_VTX_STATE_CNTL, 0);
-	OUT_ACCEL_REG(R200_RE_CNTL, 0x0);
-	/* XXX: correct?  Want it to be like RADEON_VTX_ST?_NONPARAMETRIC */
-	OUT_ACCEL_REG(R200_SE_VTE_CNTL, R200_VTX_ST_DENORMALIZED);
-	OUT_ACCEL_REG(R200_SE_VAP_CNTL, R200_VAP_FORCE_W_TO_ONE |
-	    R200_VAP_VF_MAX_VTX_NUM);
-	FINISH_ACCEL();
-    } else {
-	BEGIN_ACCEL(2);
-        if ((info->ChipFamily == CHIP_FAMILY_RADEON) ||
-            (info->ChipFamily == CHIP_FAMILY_RV200))
-            OUT_ACCEL_REG(RADEON_SE_CNTL_STATUS, 0);
-        else
-            OUT_ACCEL_REG(RADEON_SE_CNTL_STATUS, RADEON_TCL_BYPASS);
-	OUT_ACCEL_REG(RADEON_SE_COORD_FMT,
-	    RADEON_VTX_XY_PRE_MULT_1_OVER_W0 |
-	    RADEON_VTX_ST0_NONPARAMETRIC |
-	    RADEON_VTX_ST1_NONPARAMETRIC |
-	    RADEON_TEX1_W_ROUTING_USE_W0);
-	FINISH_ACCEL();
-    }
-
-    BEGIN_ACCEL(3);
-    OUT_ACCEL_REG(RADEON_RE_TOP_LEFT, 0);
-    OUT_ACCEL_REG(RADEON_RE_WIDTH_HEIGHT, 0x07ff07ff);
-    OUT_ACCEL_REG(RADEON_SE_CNTL, RADEON_DIFFUSE_SHADE_GOURAUD |
-				  RADEON_BFACE_SOLID | 
-				  RADEON_FFACE_SOLID |
-				  RADEON_VTX_PIX_CENTER_OGL |
-				  RADEON_ROUND_MODE_ROUND |
-				  RADEON_ROUND_PREC_4TH_PIX);
-    FINISH_ACCEL();
-}
-
 static Bool FUNC_NAME(R100SetupTexture)(
 	ScrnInfoPtr pScrn,
 	CARD32 format,
@@ -424,14 +383,19 @@ static Bool FUNC_NAME(R100SetupTexture)(
     RADEONInfoPtr info = RADEONPTR(pScrn);
     CARD8 *dst;
     CARD32 tex_size = 0, txformat;
-    int dst_pitch, offset, size, i, tex_bytepp;
+    int dst_pitch, offset, size, tex_bytepp;
 #ifdef ACCEL_CP
-    CARD32 buf_pitch;
+    CARD32 buf_pitch, dst_pitch_off;
+    int x, y;
     unsigned int hpass;
     CARD8 *tmp_dst;
 #endif
     ACCEL_PREAMBLE();
 
+    /* render repeat is broken - fix in stable tree by falling back */
+    if (flags & XAA_RENDER_REPEAT)
+	return FALSE;
+
     if ((width > 2048) || (height > 2048))
 	return FALSE;
 
@@ -471,12 +435,15 @@ static Bool FUNC_NAME(R100SetupTexture)(
 
 #ifdef ACCEL_CP
 
+    RADEONHostDataParams( pScrn, dst, dst_pitch, tex_bytepp, &dst_pitch_off, &x, &y );
+
     while ( height )
     {
     	tmp_dst = RADEONHostDataBlit( pScrn, tex_bytepp, width,
-							dst_pitch, &buf_pitch,
-							&dst, &height, &hpass );
-	RADEONHostDataBlitCopyPass( tmp_dst, src, hpass, buf_pitch, src_pitch );
+				      dst_pitch_off, &buf_pitch,
+				      x, &y, &height, &hpass );
+	RADEONHostDataBlitCopyPass( pScrn, tex_bytepp, tmp_dst, src,
+				    hpass, buf_pitch, src_pitch );
 	src += hpass * src_pitch;
     }
 
@@ -485,12 +452,10 @@ static Bool FUNC_NAME(R100SetupTexture)(
 
 #else
 
-    i = height;
-
     if (info->accel->NeedToSync)
 	info->accel->Sync(pScrn);
 
-    while(i--) {
+    while (height--) {
 	memcpy(dst, src, width * tex_bytepp);
 	src += src_pitch;
 	dst += dst_pitch;
@@ -542,8 +507,8 @@ FUNC_NAME(R100SetupForCPUToScreenAlphaTexture) (
     if (blend_cntl == 0)
 	return FALSE;
 
-    if (!info->RenderInited3D)
-	RadeonInit3DEngine(pScrn);
+    if (!info->XInited3D)
+	RADEONInit3DEngine(pScrn);
 
     if (!FUNC_NAME(R100SetupTexture)(pScrn, maskFormat, alphaPtr, alphaPitch,
 				     width, height, flags))
@@ -593,8 +558,8 @@ FUNC_NAME(R100SetupForCPUToScreenTexture) (
     if (blend_cntl == 0)
 	return FALSE;
     
-    if (!info->RenderInited3D)
-	RadeonInit3DEngine(pScrn);
+    if (!info->XInited3D)
+	RADEONInit3DEngine(pScrn);
 
     if (!FUNC_NAME(R100SetupTexture)(pScrn, srcFormat, texPtr, texPitch, width,
 				     height, flags))
@@ -755,14 +720,19 @@ static Bool FUNC_NAME(R200SetupTexture)(
     RADEONInfoPtr info = RADEONPTR(pScrn);
     CARD8 *dst;
     CARD32 tex_size = 0, txformat;
-    int dst_pitch, offset, size, i, tex_bytepp;
+    int dst_pitch, offset, size, tex_bytepp;
 #ifdef ACCEL_CP
-    CARD32 buf_pitch;
+    CARD32 buf_pitch, dst_pitch_off;
+    int x, y;
     unsigned int hpass;
     CARD8 *tmp_dst;
 #endif
     ACCEL_PREAMBLE();
 
+    /* render repeat is broken - fix in stable tree by falling back */
+    if (flags & XAA_RENDER_REPEAT)
+	return FALSE;
+
     if ((width > 2048) || (height > 2048))
 	return FALSE;
 
@@ -802,12 +772,15 @@ static Bool FUNC_NAME(R200SetupTexture)(
 
 #ifdef ACCEL_CP
 
+    RADEONHostDataParams( pScrn, dst, dst_pitch, tex_bytepp, &dst_pitch_off, &x, &y );
+
     while ( height )
     {
         tmp_dst = RADEONHostDataBlit( pScrn, tex_bytepp, width,
-							dst_pitch, &buf_pitch,
-							&dst, &height, &hpass );
-	RADEONHostDataBlitCopyPass( tmp_dst, src, hpass, buf_pitch, src_pitch );
+				      dst_pitch_off, &buf_pitch,
+				      x, &y, &height, &hpass );
+	RADEONHostDataBlitCopyPass( pScrn, tex_bytepp, tmp_dst, src,
+				    hpass, buf_pitch, src_pitch );
 	src += hpass * src_pitch;
     }
 
@@ -816,11 +789,10 @@ static Bool FUNC_NAME(R200SetupTexture)(
 
 #else
 
-    i = height;
     if (info->accel->NeedToSync)
 	info->accel->Sync(pScrn);
 
-    while(i--) {
+    while (height--) {
 	memcpy(dst, src, width * tex_bytepp);
 	src += src_pitch;
 	dst += dst_pitch;
@@ -873,8 +845,8 @@ FUNC_NAME(R200SetupForCPUToScreenAlphaTexture) (
     if (blend_cntl == 0)
 	return FALSE;
 
-    if (!info->RenderInited3D)
-	RadeonInit3DEngine(pScrn);
+    if (!info->XInited3D)
+	RADEONInit3DEngine(pScrn);
 
     if (!FUNC_NAME(R200SetupTexture)(pScrn, maskFormat, alphaPtr, alphaPitch,
 				     width, height, flags))
@@ -925,8 +897,8 @@ FUNC_NAME(R200SetupForCPUToScreenTexture) (
     if (blend_cntl == 0)
 	return FALSE;
 
-    if (!info->RenderInited3D)
-	RadeonInit3DEngine(pScrn);
+    if (!info->XInited3D)
+	RADEONInit3DEngine(pScrn);
 
     if (!FUNC_NAME(R200SetupTexture)(pScrn, srcFormat, texPtr, texPitch, width,
 				     height, flags))
@@ -1074,4 +1046,4 @@ FUNC_NAME(R200SubsequentCPUToScreenTexture) (
 }
 
 #undef FUNC_NAME
-
+#endif /* USE_XAA */
